@@ -30,6 +30,7 @@ class ClientWindowBase(ClientWidgetBase):
     def __init__(self, client, group_leader, wid, x, y, w, h, metadata, override_redirect, client_properties, border):
         log("%s%s", type(self), (client, group_leader, wid, x, y, w, h, metadata, override_redirect, client_properties))
         ClientWidgetBase.__init__(self, client, wid, metadata.boolget("has-alpha"))
+        self._iconified = False
         self._override_redirect = override_redirect
         self.group_leader = group_leader
         self._pos = (x, y)
@@ -107,8 +108,8 @@ class ClientWindowBase(ClientWidgetBase):
 
     def update_metadata(self, metadata):
         #normalize window-type:
-        window_type = metadata.strlistget("window-type")
-        if window_type is not None:
+        if "window-type" in metadata:
+            window_type = metadata.strlistget("window-type")
             #normalize the window type for servers that don't do "generic_window_types"
             window_type = [x.replace("_NET_WM_WINDOW_TYPE_", "").replace("_NET_WM_TYPE_", "") for x in window_type]
             metadata["window-type"] = window_type
@@ -117,12 +118,16 @@ class ClientWindowBase(ClientWidgetBase):
         if not self.is_realized():
             self.set_wmclass(*self._metadata.strlistget("class-instance",
                                                  ("xpra", "Xpra")))
-        self.set_metadata(metadata)
+        try:
+            self.set_metadata(metadata)
+        except Exception, e:
+            log.warn("failed to set window metadata to '%s': %s", metadata, e)
 
     def set_metadata(self, metadata):
+        log("set_metadata(%s)", metadata)
         if "title" in metadata:
             try:
-                title = u(self._client.title)
+                title = u(self._client.title).replace("\0", "")
                 if title.find("@")>=0:
                     #perform metadata variable substitutions:
                     default_values = {"title" : u("<untitled window>"),
@@ -218,14 +223,30 @@ class ClientWindowBase(ClientWidgetBase):
 
         if "maximized" in metadata:
             maximized = metadata.boolget("maximized")
-            if maximized:
-                self.maximize()
-            else:
-                self.unmaximize()
+            if maximized!=self._maximized:
+                self._maximized = maximized
+                if maximized:
+                    self.maximize()
+                else:
+                    self.unmaximize()
 
         if "fullscreen" in metadata:
             fullscreen = metadata.boolget("fullscreen")
-            self.set_fullscreen(fullscreen)
+            if self._fullscreen is None or self._fullscreen!=fullscreen:
+                self._fullscreen = fullscreen
+                self.set_fullscreen(fullscreen)
+
+        if "iconic" in metadata:
+            iconified = metadata.boolget("iconic")
+            if self._iconified!=iconified:
+                self._iconified = iconified
+                if self._iconified:
+                    self.iconify()
+                else:
+                    self.deiconify()
+
+        if "decorations" in metadata:
+            self.set_decorated(metadata.boolget("decorations"))
 
 
     def set_window_type(self, window_types):
@@ -287,7 +308,7 @@ class ClientWindowBase(ClientWidgetBase):
     def can_have_spinner(self):
         if self._backing is None:
             return False
-        window_types = self._metadata.get("window-type")
+        window_types = self._metadata.strlistget("window-type")
         if not window_types:
             return False
         return ("NORMAL" in window_types) or \
