@@ -14,13 +14,14 @@ required_extensions = ["GL_ARB_texture_rectangle", "GL_ARB_vertex_program"]
 
 
 WHITELIST = {}
-BLACKLIST = {"vendor" : ["nouveau", "Humper", "VMware, Inc."]}
+BLACKLIST = {"vendor" : ["nouveau", "Humper", "VMware, Inc."],
+             "renderer" : ["Software Rasterizer"]}
 if False:
     #for testing:
     BLACKLIST["vendor"].append("NVIDIA Corporation")
     WHITELIST["renderer"] = ["GeForce GTX 760/PCIe/SSE2"]
     
-if sys.platform.startswith("darwin"): 
+if sys.platform.startswith("darwin") or sys.platform.startswith("win"):
     #crashes were reported with the Intel driver on OSX 
     BLACKLIST["vendor"].append("Intel Inc.")
 
@@ -245,7 +246,7 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             glViewport, glMatrixMode, glLoadIdentity, glOrtho, \
             glEnableClientState, glGenTextures, glDisable, \
             glBindTexture, glPixelStorei, glEnable, glBegin, glFlush, \
-            glTexParameteri, \
+            glTexParameteri, glTexEnvi, glHint, glBlendFunc, glLineStipple, \
             glTexImage2D, \
             glMultiTexCoord2i, \
             glVertex2i, glEnd
@@ -253,7 +254,7 @@ def check_GL_support(gldrawable, glcontext, min_texture_size=0, force_enable=Fal
             glViewport, glMatrixMode, glLoadIdentity, glOrtho, \
             glEnableClientState, glGenTextures, glDisable, \
             glBindTexture, glPixelStorei, glEnable, glBegin, glFlush, \
-            glTexParameteri, \
+            glTexParameteri, glTexEnvi, glHint, glBlendFunc, glLineStipple, \
             glTexImage2D, \
             glMultiTexCoord2i, \
             glVertex2i, glEnd)
