@@ -13,12 +13,15 @@ import gtk.gdkgl, gtk.gtkgl         #@UnresolvedImport
 assert gtk.gdkgl is not None and gtk.gtkgl is not None
 import gobject
 
+import time, math
+
 from xpra.log import Logger
 log = Logger("opengl", "paint")
 OPENGL_DEBUG = os.environ.get("XPRA_OPENGL_DEBUG", "0")=="1"
 
 from xpra.os_util import memoryview_to_bytes 
 from xpra.codecs.codec_constants import get_subsampling_divs
+from xpra.gtk_common.gtk_spinner import cv
 from xpra.client.gl.gl_check import get_DISPLAY_MODE, GL_ALPHA_SUPPORTED, is_pyopengl_memoryview_safe
 from xpra.client.gl.gl_colorspace_conversions import YUV2RGB_shader, RGBP2RGB_shader
 from xpra.client.gtk2.window_backing import GTK2WindowBacking, fire_paint_callbacks
@@ -28,13 +31,14 @@ from OpenGL.GL import \
     GL_UNPACK_ROW_LENGTH, GL_UNPACK_ALIGNMENT, \
     GL_TEXTURE_MAG_FILTER, GL_TEXTURE_MIN_FILTER, GL_NEAREST, \
     GL_UNSIGNED_BYTE, GL_LUMINANCE, GL_LINEAR, \
-    GL_TEXTURE0, GL_TEXTURE1, GL_TEXTURE2, GL_QUADS, GL_LINE_LOOP, GL_COLOR_BUFFER_BIT, \
+    GL_TEXTURE0, GL_TEXTURE1, GL_TEXTURE2, GL_QUADS, GL_POLYGON, GL_LINE_LOOP, GL_COLOR_BUFFER_BIT, \
     GL_TEXTURE_WRAP_S, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER, \
     GL_DONT_CARE, GL_TRUE, GL_DEPTH_TEST, \
     GL_RGB, GL_RGBA, GL_BGR, GL_BGRA, \
     GL_BLEND, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA, \
     GL_TEXTURE_MAX_LEVEL, GL_TEXTURE_2D, \
     glBlendFunc, \
+    glTexEnvi, GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE, \
     glActiveTexture, glTexSubImage2D, \
     glGetString, glViewport, glMatrixMode, glLoadIdentity, glOrtho, \
     glGenTextures, glDisable, \
@@ -200,6 +204,7 @@ class GLPixmapBacking(GTK2WindowBacking):
         self.debug_setup = False
         self.border = None
         self.paint_screen = False
+        self.paint_spinner = False
         self.draw_needs_refresh = False
         self.offscreen_fbo = None
 
@@ -414,6 +419,7 @@ class GLPixmapBacking(GTK2WindowBacking):
             # support alpha channel if present:
             glEnablei(GL_BLEND, self.textures[TEX_FBO])
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)
+        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE)
         glBegin(GL_QUADS)
         glTexCoord2i(0, h)
         glVertex2i(0, 0)
@@ -424,16 +430,36 @@ class GLPixmapBacking(GTK2WindowBacking):
         glTexCoord2i(w, h)
         glVertex2i(w, 0)
         glEnd()
+        glDisable(GL_TEXTURE_RECTANGLE_ARB)
+
+        if self.paint_spinner:
+            #add spinner:
+            dim = min(w/3.0, h/3.0)
+            t = time.time()
+            count = int(t*4.0)
+            bx = w//2
+            by = h//2
+            for i in range(8):      #8 lines
+                glBegin(GL_POLYGON)
+                c = cv.trs[count%8][i]
+                glColor4f(c, c, c, 1)
+                mi1 = math.pi*i/4-math.pi/16
+                mi2 = math.pi*i/4+math.pi/16
+                glVertex2i(int(bx+math.sin(mi1)*10), int(by+math.cos(mi1)*10))
+                glVertex2i(int(bx+math.sin(mi1)*dim), int(by+math.cos(mi1)*dim))
+                glVertex2i(int(bx+math.sin(mi2)*dim), int(by+math.cos(mi2)*dim))
+                glVertex2i(int(bx+math.sin(mi2)*10), int(by+math.cos(mi2)*10))
+                glEnd()
 
         #if desired, paint window border
         if self.border and self.border.shown:
             glDisable(GL_TEXTURE_RECTANGLE_ARB)
             #double size since half the line will be off-screen
             glLineWidth(self.border.size*2)
-            glColor4f(self.border.red, self.border.green, self.border.blue, self.border.alpha)
             glBegin(GL_LINE_LOOP)
-            for x,y in ((0, 0), (w, 0), (w, h), (0, h)):
-                glVertex2i(x, y)
+            glColor4f(self.border.red, self.border.green, self.border.blue, self.border.alpha)
+            for px,py in ((0, 0), (w, 0), (w, h), (0, h)):
+                glVertex2i(px, py)
             glEnd()
             #reset color to default
             glColor4f(1.0, 1.0, 1.0, 1.0)
