@@ -75,6 +75,9 @@ def make_window_metadata(window, propname, generic_window_types=False, client_su
         if hints is not None:
             hints_metadata = hints.to_dict()
         return {"size-constraints": hints_metadata}
+    elif propname in ("iconic", ):
+        #always send this one when requested
+        return {propname : bool(window.get_property(propname))}
     elif propname == "class-instance":
         c_i = window.get_property("class-instance")
         if c_i is None:
@@ -103,7 +106,7 @@ def make_window_metadata(window, propname, generic_window_types=False, client_su
     elif propname == "window-type":
         wts = get_generic_window_type(window, strip_net=generic_window_types)
         return {"window-type" : wts}
-    elif propname in ("has-alpha", "override-redirect", "tray", "modal", "fullscreen", "maximized"):
+    elif propname in ("has-alpha", "override-redirect", "tray", "modal", "fullscreen", "maximized", "decorations"):
         return {propname : window.get_property(propname)}
     elif propname in ("role", "opacity"):
         v = window.get_property(propname)
@@ -221,7 +224,7 @@ class ServerSource(object):
         self.speaker_codecs = speaker_codecs
         self.supports_microphone = supports_microphone
         self.microphone_codecs = microphone_codecs
-        self.sound_source_sequence = -1
+        self.sound_source_sequence = 0
         self.sound_source = None
         self.sound_sink = None
 
@@ -308,6 +311,7 @@ class ServerSource(object):
         self.pointer_grabs = False
         self.randr_notify = False
         self.named_cursors = False
+        self.window_initiate_moveresize = False
         self.clipboard_enabled = False
         self.clipboard_notifications = False
         self.clipboard_set_enabled = False
@@ -533,6 +537,7 @@ class ServerSource(object):
         self.send_notifications = c.boolget("notifications")
         self.randr_notify = c.boolget("randr_notify")
         self.clipboard_enabled = c.boolget("clipboard", True)
+        self.window_initiate_moveresize = c.boolget("window.initiate-moveresize")
         self.clipboard_notifications = c.boolget("clipboard.notifications")
         self.clipboard_set_enabled = c.boolget("clipboard.set_enabled")
         self.share = c.boolget("share")
@@ -710,17 +715,19 @@ class ServerSource(object):
             assert self.supports_speaker, "cannot send sound: support not enabled on the server"
             assert self.sound_source is None, "a sound source already exists"
             assert self.sound_receive, "cannot send sound: support is not enabled on the client"
-            self.sound_source = start_sending_sound(codec, volume, self.sound_decoders, self.microphone_codecs, self.pulseaudio_server, self.pulseaudio_id)
-            soundlog("start_sending_sound() sound source=%s", self.sound_source)
-            if self.sound_source:
+            ss = start_sending_sound(codec, volume, self.sound_decoders, self.microphone_codecs, self.pulseaudio_server, self.pulseaudio_id)
+            self.sound_source = ss
+            soundlog("start_sending_sound() sound source=%s", ss)
+            if ss:
+                ss.sequence = self.sound_source_sequence
                 if self.server_driven:
                     #tell the client this is the start:
-                    self.send("sound-data", self.sound_source.codec, "",
+                    self.send("sound-data", ss.codec, "",
                               {"start-of-stream"    : True,
-                               "codec"              : self.sound_source.codec,
-                               "sequence"           : self.sound_source_sequence})
-                self.sound_source.connect("new-buffer", self.new_sound_buffer)
-                self.sound_source.start()
+                               "codec"              : ss.codec,
+                               "sequence"           : ss.sequence})
+                ss.connect("new-buffer", self.new_sound_buffer)
+                ss.start()
         except Exception, e:
             log.error("error setting up sound: %s", e)
 
@@ -731,7 +738,8 @@ class ServerSource(object):
             self.sound_source = None
             if self.server_driven:
                 #tell the client this is the end:
-                self.send("sound-data", ss.codec, "", {"end-of-stream" : True})
+                self.send("sound-data", ss.codec, "", {"end-of-stream" : True,
+                                                       "sequence"      : ss.sequence})
             def stop_sending_sound_thread(*args):
                 soundlog("stop_sending_sound_thread(%s)", args)
                 ss.cleanup()
@@ -741,11 +749,13 @@ class ServerSource(object):
     def new_sound_buffer(self, sound_source, data, metadata):
         soundlog("new_sound_buffer(%s, %s, %s) source=%s, suspended=%s, sequence=%s",
                  sound_source, len(data or []), metadata, self.sound_source, self.suspended, self.sound_source_sequence)
-        if self.sound_source is None or self.is_closed():
+        if self.sound_source!=sound_source or self.is_closed():
             return
-        if self.sound_source_sequence>0:
-            metadata["sequence"] = self.sound_source_sequence
-        self.send("sound-data", self.sound_source.codec, Compressed(self.sound_source.codec, data), metadata)
+        if sound_source.sequence<self.sound_source_sequence:
+            return
+        if sound_source.sequence>=0:
+            metadata["sequence"] = sound_source.sequence
+        self.send("sound-data", sound_source.codec, Compressed(sound_source.codec, data), metadata)
 
     def stop_receiving_sound(self):
         ss = self.sound_sink
@@ -811,7 +821,7 @@ class ServerSource(object):
                 return False
             self.timeout_add(100, fadeout)
         elif action=="new-sequence":
-            self.sound_source_sequence = args[0]
+            self.sound_source_sequence = int(args[0])
             return "new sequence is %s" % self.sound_source_sequence
         #elif action=="quality":
         #    assert self.sound_source
@@ -831,6 +841,10 @@ class ServerSource(object):
             log.info("sound codec changed from %s to %s", self.sound_sink.codec, codec)
             self.sound_sink.cleanup()
             self.sound_sink = None
+        if metadata.get("end-of-stream"):
+            soundlog("client sent end-of-stream, closing sound pipeline")
+            self.stop_receiving_sound(False)
+            return
         if not self.sound_sink:
             try:
                 def sound_sink_error(*args):
@@ -1347,6 +1361,13 @@ class ServerSource(object):
             return True
         return False
 
+
+    def initiate_moveresize(self, wid, window, x_root, y_root, direction, button, source_indication):
+        if not self.can_send_window(window) or not self.window_initiate_moveresize:
+            return
+        log("initiate_moveresize sending to %s", self)
+        self.send("initiate-moveresize", wid, x_root, y_root, direction, button, source_indication)
+
     def or_window_geometry(self, wid, window, x, y, w, h):
         if not self.can_send_window(window):
             return
