@@ -14,7 +14,7 @@ from xpra.codecs.codec_constants import get_avutil_enum_from_colorspace, get_sub
                                         TransientCodecException, RGB_FORMATS, PIXEL_SUBSAMPLING, LOSSY_PIXEL_FORMATS
 from xpra.server.window_source import WindowSource, STRICT_MODE, AUTO_REFRESH_SPEED, AUTO_REFRESH_QUALITY
 from xpra.server.video_subregion import VideoSubregion
-from xpra.codecs.loader import PREFERED_ENCODING_ORDER
+from xpra.codecs.loader import PREFERED_ENCODING_ORDER, EDGE_ENCODING_ORDER
 from xpra.util import updict
 from xpra.log import Logger
 
@@ -121,6 +121,7 @@ class WindowVideoSource(WindowSource):
         self.full_csc_modes = {}                            #for 0.12 onwards: per encoding lists
         self.video_encodings = []
         self.non_video_encodings = []
+        self.edge_encoding = None
 
 
     def get_client_info(self):
@@ -155,6 +156,8 @@ class WindowVideoSource(WindowSource):
             info["encoder"] = ve.get_type()
             up("encoder", ve.get_info())
         up("encoding.pipeline_param", self.get_pipeline_info())
+        info["encodings.non-video"] = self.non_video_encodings
+        info["encodings.edge"] = self.edge_encoding or ""
         if self._last_pipeline_check>0:
             info["encoding.pipeline_last_check"] = int(1000*(time.time()-self._last_pipeline_check))
         lps = self.last_pipeline_scores
@@ -201,7 +204,7 @@ class WindowVideoSource(WindowSource):
         self.cleanup_codecs()
 
     def cleanup_codecs(self):
-        """ Video encoders (x264, nvenc and vpx) and their csc helpers
+        """ Video encoders (ie: x264, vpx) and their csc helpers
             require us to run cleanup code to free the memory they use.
             But some cleanups may be slow, so run them in a worker thread.
         """
@@ -253,7 +256,11 @@ class WindowVideoSource(WindowSource):
         #encodings may have changed, so redo this:
         nv_common = (set(self.server_core_encodings) & set(self.core_encodings)) - set(self.video_encodings)
         self.non_video_encodings = [x for x in PREFERED_ENCODING_ORDER if x in nv_common]
-        log("do_set_client_properties(%s) csc_modes=%s, full_csc_modes=%s, video_scaling=%s, video_subregion=%s, uses_swscale=%s, non_video_encodings=%s", properties, self.csc_modes, self.full_csc_modes, self.supports_video_scaling, self.supports_video_subregion, self.uses_swscale, self.non_video_encodings)
+        try:
+            self.edge_encoding = [x for x in EDGE_ENCODING_ORDER if x in self.non_video_encodings][0]
+        except:
+            self.edge_encoding = None
+        log("do_set_client_properties(%s) csc_modes=%s, full_csc_modes=%s, video_scaling=%s, video_subregion=%s, uses_swscale=%s, non_video_encodings=%s, edge_encoding=%s", properties, self.csc_modes, self.full_csc_modes, self.supports_video_scaling, self.supports_video_subregion, self.uses_swscale, self.non_video_encodings, self.edge_encoding)
 
     def get_best_encoding_impl_default(self):
         return self.get_best_encoding_video
@@ -332,14 +339,17 @@ class WindowVideoSource(WindowSource):
         #if we're here, then the window has no alpha (or the client cannot handle alpha)
         #and we can ignore the current encoding
         options = options or self.non_video_encodings
-        if pixel_count<self._rgb_auto_threshold and "rgb24" in options:
+        if pixel_count<self._rgb_auto_threshold:
             #high speed and high quality, rgb is still good
-            return "rgb24"
+            if "rgb24" in options:
+                return "rgb24"
+            if "rgb32" in options:
+                return "rgb32"
         #use sliding scale for lossless threshold
         #(high speed favours switching to lossy sooner)
         #take into account how many pixels need to be encoder:
         #more pixels means we switch to lossless more easily
-        lossless_q = self._lossless_threshold_base + self._lossless_threshold_pixel_boost * pixel_count / (ww*wh)
+        lossless_q = min(100, self._lossless_threshold_base + self._lossless_threshold_pixel_boost * pixel_count / (ww*wh))
         if quality<lossless_q:
             #lossy options:
             if "jpeg" in options:
@@ -360,8 +370,11 @@ class WindowVideoSource(WindowSource):
                 max_webp = 1024*1024 * (200-quality)/100 * speed/100
                 if pixel_count<max_webp:
                     return "webp"
-            if "rgb24" in options and speed>75:
-                return "rgb24"
+            if speed>75:
+                if "rgb24" in options:
+                    return "rgb24"
+                if "rgb32" in options:
+                    return "rgb32"
             if "png" in options:
                 return "png"
         #we failed to find a good match, default to the first of the options..
@@ -411,6 +424,12 @@ class WindowVideoSource(WindowSource):
         return q
 
 
+    def client_decode_error(self, error):
+        WindowSource.client_decode_error(self, error)
+        #maybe the stream is now corrupted..
+        self.cleanup_codecs()
+
+
     def get_refresh_exclude(self):
         #exclude video region (if any) from lossless refresh:
         return self.video_subregion.rectangle
@@ -568,21 +587,24 @@ class WindowVideoSource(WindowSource):
 
 
     def process_damage_region(self, damage_time, window, x, y, w, h, coding, options):
-        WindowSource.process_damage_region(self, damage_time, window, x, y, w, h, coding, options)
         #now figure out if we need to send edges separately:
-        dw = w - (w & self.width_mask)
-        dh = h - (h & self.height_mask)
-        if coding in self.video_encodings and (dw>0 or dh>0):
-            #no point in using get_best_encoding here, rgb24 wins
-            #(as long as the mask is small - and it is)
-            if dw>0:
-                WindowSource.process_damage_region(self, damage_time, window, x+w-dw, y, dw, h, "rgb24", options)
-            if dh>0:
-                WindowSource.process_damage_region(self, damage_time, window, x, y+h-dh, x+w, dh, "rgb24", options)
+        if coding in self.video_encodings:
+            dw = w - (w & self.width_mask)
+            dh = h - (h & self.height_mask)
+        else:
+            dw, dh = 0, 0
+        WindowSource.process_damage_region(self, damage_time, window, x, y, w-dw, h-dh, coding, options)
+        if not self.edge_encoding:
+            #we can't send the edges, no big deal
+            return
+        if dw>0:
+            WindowSource.process_damage_region(self, damage_time, window, x+w-dw, y, dw, h, self.edge_encoding, options)
+        if dh>0:
+            WindowSource.process_damage_region(self, damage_time, window, x, y+h-dh, x+w, dh, self.edge_encoding, options)
 
 
     def must_encode_full_frame(self, window, encoding):
-        return self.full_frames_only or self.is_tray or (encoding in self.video_encodings)
+        return self.full_frames_only or self.is_tray or (encoding in self.video_encodings) or not self.non_video_encodings
 
 
     def update_encoding_options(self, force_reload=False):
@@ -598,7 +620,7 @@ class WindowVideoSource(WindowSource):
         WindowSource.update_encoding_options(self, force_reload)
         log("update_encoding_options(%s) csc_encoder=%s, video_encoder=%s", force_reload, self._csc_encoder, self._video_encoder)
         if self.supports_video_subregion:
-            if self.encoding in self.video_encodings and not self.full_frames_only and not STRICT_MODE:
+            if self.encoding in self.video_encodings and not self.full_frames_only and not STRICT_MODE and len(self.non_video_encodings)>0:
                 ww, wh = self.window_dimensions
                 self.video_subregion.identify_video_subregion(ww, wh, self.statistics.damage_events_count, self.statistics.last_damage_events)
             else:
