@@ -62,6 +62,7 @@ NotifyNormal        = constants["NotifyNormal"]
 NotifyGrab          = constants["NotifyGrab"]
 NotifyUngrab        = constants["NotifyUngrab"]
 NotifyWhileGrabbed  = constants["NotifyWhileGrabbed"]
+NotifyNonlinearVirtual = constants["NotifyNonlinearVirtual"]
 GRAB_CONSTANTS = {
                   NotifyNormal          : "NotifyNormal",
                   NotifyGrab            : "NotifyGrab",
@@ -89,6 +90,10 @@ USE_XSHM = os.environ.get("XPRA_XSHM", "1")=="1"
 PROPERTIES_IGNORED = ("_NET_WM_OPAQUE_REGION", )
 
 
+def sanestr(s):
+    return (s or "").strip("\0").replace("\0", " ")
+
+
 # Todo:
 #   client focus hints
 #   _NET_WM_SYNC_REQUEST
@@ -294,6 +299,7 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
         "client-contents-changed": one_arg_signal,
         "raised"                : one_arg_signal,
         "unmanaged"             : one_arg_signal,
+        "initiate-moveresize"   : one_arg_signal,
 
         "grab"                  : one_arg_signal,
         "ungrab"                : one_arg_signal,
@@ -314,7 +320,6 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
         self._input_field = True            # The WM_HINTS input field
         self._geometry = None
         self._damage_forward_handle = None
-        self._last_wm_state_serial = 0
         self._internal_set_property("client-window", client_window)
         use_xshm = USE_XSHM and (not self.is_OR() and not self.is_tray())
         self._composite = CompositeHelper(self.client_window, False, use_xshm)
@@ -337,6 +342,7 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
     def call_setup(self):
         try:
             self._geometry = trap.call_synced(X11Window.geometry_with_border, self.client_window.xid)
+            self._read_initial_X11_properties()
         except XError, e:
             raise Unmanageable(e)
         add_event_receiver(self.client_window, self)
@@ -474,13 +480,15 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
             window_type = self._guess_window_type(transient_for)
             window_types = [gtk.gdk.atom_intern(window_type)]
         self._internal_set_property("window-type", window_types)
-        self._internal_set_property("has-alpha", self.client_window.get_depth() in (0, 32))
         self._internal_set_property("xid", self.client_window.xid)
         self._internal_set_property("pid", pget("_NET_WM_PID", "u32") or -1)
         self._internal_set_property("role", pget("WM_WINDOW_ROLE", "latin1"))
         for mutable in ["WM_NAME", "_NET_WM_NAME", "_NET_WM_WINDOW_OPACITY"]:
             self._call_property_handler(mutable)
 
+    def _read_initial_X11_properties(self):
+        self._internal_set_property("has-alpha", X11Window.get_depth(self.client_window.xid)==32)
+
 
     def _handle_opacity_change(self):
         opacity = self.prop_get("_NET_WM_WINDOW_OPACITY", "u32", True) or -1
@@ -488,13 +496,10 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
     _property_handlers["_NET_WM_WINDOW_OPACITY"] = _handle_opacity_change
 
     def _handle_title_change(self):
-        net_wm_name = self.prop_get("_NET_WM_NAME", "utf8", True)
-        if net_wm_name is not None:
-            self._internal_set_property("title", net_wm_name)
-        else:
-            # may be None
-            wm_name = self.prop_get("WM_NAME", "latin1", True)
-            self._internal_set_property("title", wm_name)
+        name = self.prop_get("_NET_WM_NAME", "utf8", True)
+        if name is None:
+            name = self.prop_get("WM_NAME", "latin1", True)
+        self._internal_set_property("title", sanestr(name))
 
     _property_handlers["WM_NAME"] = _handle_title_change
     _property_handlers["_NET_WM_NAME"] = _handle_title_change
@@ -586,29 +591,40 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
         #   _NET_WM_STATE (more fully)
         if event.message_type=="_NET_WM_STATE" and event.data and len(event.data)==5:
             atom1 = get_pyatom(event.window, event.data[1])
-            atom2 = get_pyatom(event.window, event.data[2])
             _NET_WM_STATE_REMOVE = 0
             _NET_WM_STATE_ADD = 1
             if atom1=="_NET_WM_STATE_FULLSCREEN":
                 fullscreen = event.data[0]==_NET_WM_STATE_ADD
                 log("do_xpra_client_message_event(..) setting fullscreen=%s", fullscreen)
                 self.set_property("fullscreen", fullscreen)
-            elif atom1 in ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ") and \
-                 atom2 in ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ"):
-                if self._last_wm_state_serial == event.serial:
-                    #already seen!
-                    return
-                maximized = event.data[0]==_NET_WM_STATE_ADD
-                log("do_xpra_client_message_event(%s) window maximized=%s", event, maximized)
-                self.set_property("maximized", maximized)
+            elif atom1 in ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ"):
+                atom2 = get_pyatom(event.window, event.data[2])
+                if atom1!=atom2 and atom2 in ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ"):
+                    if event.data[0]==_NET_WM_STATE_ADD:
+                        maximized = True
+                    elif event.data[0]==_NET_WM_STATE_REMOVE:
+                        maximized = False
+                    elif event.data[0]==_NET_WM_STATE_TOGGLE:
+                        maximized = not self.get_property("maximized")
+                    else:
+                        log.warn("invalid mode for _NET_WM_STATE: %s", event.data[0])
+                        return
+                    log("do_xpra_client_message_event(%s) window maximized=%s (current state=%s)", event, maximized, self.get_property("maximized"))
+                    if maximized!=self.get_property("maximized"):
+                        self.set_property("maximized", maximized)
             else:
                 log("do_xpra_client_message_event(%s) atom=%s", event, atom1)
-        if event.message_type=="_NET_ACTIVE_WINDOW" and event.data and len(event.data)==5 and event.data[0] in (0, 1):
+        elif event.message_type=="WM_CHANGE_STATE" and event.data and len(event.data)==5:
+            log("WM_CHANGE_STATE: %s", event.data[0])
+            if event.data[0]==IconicState:
+                self._internal_set_property("iconic", True)
+        elif event.message_type=="_NET_WM_MOVERESIZE" and event.data and len(event.data)==5:
+            self.emit("initiate-moveresize", event)
+        elif event.message_type=="_NET_ACTIVE_WINDOW" and event.data and len(event.data)==5 and event.data[0] in (0, 1):
             self.set_active()
             self.emit("raised", event)
         else:
             log("do_xpra_client_message_event(%s)", event)
-        self._last_wm_state_serial = event.serial
 
     def set_active(self):
         prop_set(self.client_window.get_screen().get_root_window(), "_NET_ACTIVE_WINDOW", "u32", self.client_window.xid)
@@ -617,7 +633,10 @@ class BaseWindowModel(AutoPropGObjectMixin, gobject.GObject):
     def do_xpra_focus_in_event(self, event):
         grablog("focus_in_event(%s) mode=%s, detail=%s",
             event, GRAB_CONSTANTS.get(event.mode), DETAIL_CONSTANTS.get(event.detail, event.detail))
-        self.may_emit_grab(event)
+        if event.mode==NotifyNormal and event.detail==NotifyNonlinearVirtual:
+            self.emit("raised", event)
+        else:
+            self.may_emit_grab(event)
 
     def do_xpra_focus_out_event(self, event):
         grablog("focus_out_event(%s) mode=%s, detail=%s",
@@ -786,6 +805,10 @@ class WindowModel(BaseWindowModel):
         "owner": (gobject.TYPE_PYOBJECT,
                   "Owner", "",
                   gobject.PARAM_READABLE),
+        "decorations": (gobject.TYPE_BOOLEAN,
+                       "Should the window decorations be shown", "",
+                       True,
+                       gobject.PARAM_READABLE),
         }
     __gsignals__ = {
         # X11 bell event:
@@ -823,7 +846,7 @@ class WindowModel(BaseWindowModel):
 
         self.connect("notify::iconic", self._handle_iconic_update)
 
-        self.property_names += ["title", "icon-title", "size-hints", "class-instance", "icon", "client-machine", "modal"]
+        self.property_names += ["title", "icon-title", "size-hints", "class-instance", "icon", "client-machine", "modal", "iconic", "decorations"]
         self.call_setup()
 
     def setup(self):
@@ -1185,13 +1208,11 @@ class WindowModel(BaseWindowModel):
     _property_handlers["WM_NORMAL_HINTS"] = _handle_wm_normal_hints
 
     def _handle_icon_title_change(self):
-        net_wm_icon_name = self.prop_get("_NET_WM_ICON_NAME", "utf8", True)
-        if net_wm_icon_name is not None:
-            self._internal_set_property("icon-title", net_wm_icon_name)
-        else:
-            # may be None
-            wm_icon_name = self.prop_get("WM_ICON_NAME", "latin1", True)
-            self._internal_set_property("icon-title", wm_icon_name)
+        icon_name = self.prop_get("_NET_WM_ICON_NAME", "utf8", True)
+        log("_NET_WM_ICON_NAME=%s", icon_name)
+        if icon_name is None:
+            icon_name = self.prop_get("WM_ICON_NAME", "latin1", True)
+        self._internal_set_property("icon-title", sanestr(icon_name))
 
     _property_handlers["WM_ICON_NAME"] = _handle_icon_title_change
     _property_handlers["_NET_WM_ICON_NAME"] = _handle_icon_title_change
@@ -1208,6 +1229,13 @@ class WindowModel(BaseWindowModel):
     _property_handlers["_NET_WM_STRUT"] = _handle_wm_strut
     _property_handlers["_NET_WM_STRUT_PARTIAL"] = _handle_wm_strut
 
+    def _handle_motif_wm_hints(self):
+        #motif_hints = self.prop_get("_MOTIF_WM_HINTS", "motif-hints")
+        motif_hints = prop_get(self.client_window, "_MOTIF_WM_HINTS", "motif-hints", ignore_errors=False, raise_xerrors=True)
+        log("_handle_motif_wm_hints() motif_hints=%s", motif_hints)
+        self._internal_set_property("decorations", (motif_hints is None) or bool(motif_hints.decorations))
+    _property_handlers["_MOTIF_WM_HINTS"] = _handle_motif_wm_hints
+
     def _handle_net_wm_icon(self):
         log("_NET_WM_ICON changed on %#x, re-reading", self.client_window.xid)
         surf = self.prop_get("_NET_WM_ICON", "icon")
@@ -1241,21 +1269,25 @@ class WindowModel(BaseWindowModel):
         self._internal_set_property("requested-position", (geometry[0], geometry[1]))
         self._internal_set_property("requested-size", (geometry[2], geometry[3]))
 
-        def set_class_instance(s):
-            try:
-                parts = class_instance.split("\0")
-                if len(parts)!=3:
-                    return  False
-                (c, i, _) = parts
-                self._internal_set_property("class-instance", (c, i))
-                return  True
-            except ValueError:
-                log.warn("Malformed WM_CLASS: %s, ignoring", class_instance)
-                return  False
-        class_instance = pget("WM_CLASS", "latin1")
-        if class_instance:
-            if not set_class_instance(class_instance):
-                set_class_instance(pget("WM_CLASS", "utf8"))
+        #try using XGetClassHint:
+        class_instance = X11Window.getClassHint(self.client_window.xid)
+        if class_instance is None:
+            #fallback to reading WM_CLASS:
+            def get_wm_class_prop(ptype):
+                class_instance = pget("WM_CLASS", ptype)
+                if not class_instance:
+                    return None
+                try:
+                    parts = class_instance.split("\0")
+                    if len(parts)!=3:
+                        return None
+                    (c, i, _) = parts
+                    return  (c, i)
+                except ValueError:
+                    log.warn("Malformed WM_CLASS: %s, ignoring", class_instance)
+                    return None
+            class_instance = get_wm_class_prop("latin1") or get_wm_class_prop("utf8")
+        self._internal_set_property("class-instance", class_instance)
 
         protocols = pget("WM_PROTOCOLS", ["atom"])
         if protocols is None:
@@ -1290,9 +1322,12 @@ class WindowModel(BaseWindowModel):
         modal = (net_wm_state is not None) and ("_NET_WM_STATE_MODAL" in net_wm_state)
         self._internal_set_property("modal", modal)
 
+        #the default value is True, but is not being honoured! (so we force it here)
+        self._internal_set_property("decorations", True)
+
         for mutable in ["WM_HINTS", "WM_NORMAL_HINTS",
                         "WM_ICON_NAME", "_NET_WM_ICON_NAME",
-                        "_NET_WM_STRUT", "_NET_WM_STRUT_PARTIAL"]:
+                        "_NET_WM_STRUT", "_NET_WM_STRUT_PARTIAL", "_MOTIF_WM_HINTS"]:
             self._call_property_handler(mutable)
         for mutable in ["_NET_WM_ICON"]:
             try:
@@ -1321,45 +1356,59 @@ class WindowModel(BaseWindowModel):
     #      accesses the "state" set directly.  This is done by overriding
     #      do_set_property and do_get_property.
     _state_properties = {
-        "attention-requested": "_NET_WM_STATE_DEMANDS_ATTENTION",
-        "fullscreen": "_NET_WM_STATE_FULLSCREEN",
+        "attention-requested"   : ("_NET_WM_STATE_DEMANDS_ATTENTION", ),
+        "fullscreen"            : ("_NET_WM_STATE_FULLSCREEN", ),
+        "maximized"             : ("_NET_WM_STATE_MAXIMIZED_VERT", "_NET_WM_STATE_MAXIMIZED_HORZ"),
         }
 
     _state_properties_reversed = {}
-    for k, v in _state_properties.iteritems():
-        _state_properties_reversed[v] = k
+    for k, states in _state_properties.iteritems():
+        for x in states:
+            _state_properties_reversed[x] = k
 
-    def _state_add(self, state_name):
+    def _state_add(self, *state_names):
         curr = set(self.get_property("state"))
-        if state_name not in curr:
-            curr.add(state_name)
-            self._internal_set_property("state", ImmutableSet(curr))
-            if state_name in self._state_properties_reversed:
-                self.notify(self._state_properties_reversed[state_name])
+        add = [s for s in state_names if s not in curr]
+        if add:
+            for x in add:
+                curr.add(x)
+            self._internal_set_property("state", frozenset(curr))
+            self._state_notify(add)
 
-    def _state_remove(self, state_name):
+    def _state_remove(self, *state_names):
         curr = set(self.get_property("state"))
-        if state_name in curr:
-            curr.discard(state_name)
-            self._internal_set_property("state", ImmutableSet(curr))
-            if state_name in self._state_properties_reversed:
-                self.notify(self._state_properties_reversed[state_name])
+        discard = [s for s in state_names if s in curr]
+        if discard:
+            for x in discard:
+                curr.discard(x)
+            self._internal_set_property("state", frozenset(curr))
+            self._state_notify(discard)
+
+    def _state_notify(self, state_names):
+        notify_props = set()
+        for x in state_names:
+            if x in self._state_properties_reversed:
+                notify_props.add(self._state_properties_reversed[x])
+        for x in list(notify_props):
+            self.notify(x)
 
     def _state_isset(self, state_name):
         return state_name in self.get_property("state")
 
     def _handle_state_changed(self, *args):
         # Sync changes to "state" property out to X property.
-        trap.swallow_synced(prop_set, self.client_window, "_NET_WM_STATE",
+        def set_wm_state():
+            prop_set(self.client_window, "_NET_WM_STATE",
                  ["atom"], self.get_property("state"))
+        trap.swallow(set_wm_state)
 
     def do_set_property(self, pspec, value):
         if pspec.name in self._state_properties:
-            state = self._state_properties[pspec.name]
+            state_names = self._state_properties[pspec.name]
             if value:
-                self._state_add(state)
+                self._state_add(*state_names)
             else:
-                self._state_remove(state)
+                self._state_remove(*state_names)
         else:
             AutoPropGObjectMixin.do_set_property(self, pspec, value)
 
@@ -1369,16 +1418,19 @@ class WindowModel(BaseWindowModel):
 
     def do_get_property(self, pspec):
         if pspec.name in self._state_properties:
-            return self._state_isset(self._state_properties[pspec.name])
+            #return True if any is set (only relevant for maximized)
+            state_names = self._state_properties[pspec.name]
+            for x in state_names:
+                if self._state_isset(x):
+                    return True
+            return False
         else:
             return AutoPropGObjectMixin.do_get_property(self, pspec)
 
 
     def _handle_iconic_update(self, *args):
         def set_state(state):
-            trap.swallow_synced(prop_set, self.client_window, "WM_STATE",
-                             ["u32"],
-                             [state, XNone])
+            prop_set(self.client_window, "WM_STATE", "state", state)
 
         if self.get_property("iconic"):
             set_state(IconicState)
