@@ -1,7 +1,7 @@
 # coding=utf8
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2015 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -11,6 +11,7 @@
 #   shape?
 #   any other interesting metadata? _NET_WM_TYPE, WM_TRANSIENT_FOR, etc.?
 
+import os
 import gtk.gdk
 import gobject
 
@@ -49,6 +50,8 @@ from xpra.os_util import StringIOClass
 from xpra.x11.x11_server_base import X11ServerBase, mouselog
 from xpra.net.compression import Compressed
 
+REPARENT_ROOT = os.environ.get("XPRA_REPARENT_ROOT", "0")=="1"
+
 
 class DesktopManager(gtk.Widget):
     def __init__(self):
@@ -110,8 +113,6 @@ class DesktopManager(gtk.Widget):
             win.maybe_recalculate_geometry_for(self)
 
     def hide_window(self, model):
-        if not model.get_property("iconic"):
-            model.set_property("iconic", True)
         self._models[model].shown = False
         model.ownership_election()
 
@@ -130,7 +131,11 @@ class DesktopManager(gtk.Widget):
             return (-1, self)
 
     def take_window(self, model, window):
-        window.reparent(self.window, 0, 0)
+        if REPARENT_ROOT:
+            parent = self.window.get_screen().get_root_window()
+        else:
+            parent = self.window
+        window.reparent(parent, 0, 0)
 
     def window_size(self, model):
         w, h = self._models[model].geom[2:4]
@@ -143,24 +148,6 @@ class DesktopManager(gtk.Widget):
                      "%sx%s vs %sx%s", w0, h0, w, h)
         return x, y
 
-    def get_transient_for(self, window, window_to_id):
-        transient_for = window.get_property("transient-for")
-        if transient_for is None:
-            return None
-        log("found transient_for=%s, xid=%#x", transient_for, transient_for.xid)
-        #try to find the model for this window:
-        for model in self._models.keys():
-            log("testing model %s: %#x", model, model.client_window.xid)
-            if model.client_window.xid==transient_for.xid:
-                wid = window_to_id.get(model)
-                log("found match, window id=%s", wid)
-                return wid
-        root = gtk.gdk.get_default_root_window()
-        if root.xid==transient_for.xid:
-            return -1       #-1 is the backwards compatible marker for root...
-        log("not found transient_for=%s, xid=%#x", transient_for, transient_for.xid)
-        return  None
-
 
 gobject.type_register(DesktopManager)
 
@@ -176,8 +163,9 @@ class XpraServer(gobject.GObject, X11ServerBase):
         X11ServerBase.__init__(self)
 
     def init(self, clobber, opts):
-        X11ServerBase.init(self, clobber, opts)
         self.xsettings_enabled = opts.xsettings
+        self.wm_name = opts.wm_name
+        X11ServerBase.init(self, clobber, opts)
 
     def x11_init(self):
         X11ServerBase.x11_init(self)
@@ -197,7 +185,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
         add_event_receiver(root, self)
 
         ### Create the WM object
-        self._wm = Wm(self.clobber)
+        self._wm = Wm(self.clobber, self.wm_name)
         self._wm.connect("new-window", self._new_window_signaled)
         self._wm.connect("bell", self._bell_signaled)
         self._wm.connect("quit", lambda _: self.quit(True))
@@ -279,7 +267,22 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
 
     def get_transient_for(self, window):
-        return self._desktop_manager.get_transient_for(window, self._window_to_id)
+        transient_for = window.get_property("transient-for")
+        log("get_transient_for window=%s, transient_for=%s", window, transient_for)
+        if transient_for is None:
+            return None
+        xid = transient_for.xid
+        log("transient_for.xid=%#x", xid)
+        for w,wid in self._window_to_id.items():
+            if w.get_property("xid")==xid:
+                log("found match, window id=%s", wid)
+                return wid
+        root = gtk.gdk.get_default_root_window()
+        if root.xid==xid:
+            log("transient-for using root")
+            return -1       #-1 is the backwards compatible marker for root...
+        log("not found transient_for=%s, xid=%#x", transient_for, xid)
+        return  None
 
     def is_shown(self, window):
         return self._desktop_manager.is_shown(window)
@@ -393,11 +396,12 @@ class XpraServer(gobject.GObject, X11ServerBase):
         window.managed_connect("client-contents-changed", self._contents_changed)
         window.managed_connect("unmanaged", self._lost_window)
         window.managed_connect("raised", self._raised_window)
+        window.managed_connect("initiate-moveresize", self._initiate_moveresize)
         window.managed_connect("grab", self._window_grab)
         window.managed_connect("ungrab", self._window_ungrab)
         return wid
 
-    _window_export_properties = ("title", "size-hints", "fullscreen", "maximized", "opacity")
+    _window_export_properties = ("title", "size-hints", "fullscreen", "maximized", "opacity", "iconic")
     def _add_new_window(self, window):
         self._add_new_window_common(window)
         for prop in self._window_export_properties:
@@ -615,12 +619,33 @@ class XpraServer(gobject.GObject, X11ServerBase):
             ss.pointer_ungrab(grab_id)
 
 
-    def _raised_window(self, window, event):
-        windowlog("raised window: %s (%s)", window, event)
+    def _initiate_moveresize(self, window, event):
+        log("initiate_moveresize(%s, %s)", window, event)
+        assert len(event.data)==5
+        #x_root, y_root, direction, button, source_indication = event.data
         wid = self._window_to_id[window]
+        for ss in self._server_sources.values():
+            ss.initiate_moveresize(wid, window, *event.data)
+
+
+    def _raised_window(self, window, event):
+        wid = self._window_to_id[window]
+        windowlog("raised window: %s (%s) wid=%s, current focus=%s", window, event, wid, self._has_focus)
+        if self._has_focus==wid:
+            return
         for ss in self._server_sources.values():
             ss.raise_window(wid, window)
 
+
+    def _set_window_state(self, proto, wid, window, new_window_state):
+        #only used for setting maximized state:
+        if "maximized" in new_window_state:
+            new_state = bool(new_window_state.get("maximized", False))
+            cur_state = bool(window.get_property("maximized"))
+            if cur_state!=new_state:
+                window.set_property("maximized", new_state)
+        #TODO: also sync "above", "below", "sticky" and "fullscreen"
+
     def _process_map_window(self, proto, packet):
         wid, x, y, width, height = packet[1:6]
         window = self._id_to_window.get(wid)
@@ -629,6 +654,8 @@ class XpraServer(gobject.GObject, X11ServerBase):
             return
         assert not window.is_OR()
         windowlog("client mapped window %s - %s, at: %s", wid, window, (x, y, width, height))
+        if len(packet)>=8:
+            self._set_window_state(proto, wid, window, packet[7])
         self._desktop_manager.configure_window(window, x, y, width, height)
         self._desktop_manager.show_window(window)
         if len(packet)>=7:
@@ -638,6 +665,7 @@ class XpraServer(gobject.GObject, X11ServerBase):
 
     def _process_unmap_window(self, proto, packet):
         wid = packet[1]
+        iconified = len(packet)>=3 and bool(packet[2])
         window = self._id_to_window.get(wid)
         if not window:
             log("cannot map window %s: already removed!", wid)
@@ -646,14 +674,12 @@ class XpraServer(gobject.GObject, X11ServerBase):
         windowlog("client unmapped window %s - %s", wid, window)
         for ss in self._server_sources.values():
             ss.unmap_window(wid, window)
+        if iconified and not window.get_property("iconic"):
+            window.set_property("iconic", True)
         self._desktop_manager.hide_window(window)
 
     def _process_configure_window(self, proto, packet):
         wid, x, y, w, h = packet[1:6]
-        if len(packet)>=7:
-            client_properties = packet[6]
-        else:
-            client_properties = {}
         resize_counter = 0
         if len(packet)>=8:
             resize_counter = packet[7]
@@ -668,11 +694,13 @@ class XpraServer(gobject.GObject, X11ServerBase):
             self._tray.move_resize(window, x, y, w, h)
         else:
             assert not window.is_OR()
+            if len(packet)>=9:
+                self._set_window_state(proto, wid, window, packet[8])
             owx, owy, oww, owh = self._desktop_manager.window_geometry(window)
             windowlog("_process_configure_window(%s) old window geometry: %s", packet[1:], (owx, owy, oww, owh))
             self._desktop_manager.configure_window(window, x, y, w, h, resize_counter)
-        if client_properties:
-            self._set_client_properties(proto, wid, window, client_properties)
+        if len(packet)>=7:
+            self._set_client_properties(proto, wid, window, packet[6])
         if window.is_tray() or (self._desktop_manager.visible(window) and (oww!=w or owh!=h)):
             self._damage(window, 0, 0, w, h)
 
